/*******************************************************************************************/
/*    IDEAL n of 1 sample size V1                                                          */
/* Program to produce sample size calculations for estimating the overall                  */
/* mean from a set of n of 1 trials                                                        */
/* A fixed and random  effects version is calculated                                       */
/* Calculation for a future patient is also given                                          */
/*******************************************************************************************/

/*******************************************************************************************/
/*  Author: Stephen Senn                                                                   */
/*******************************************************************************************/

/*******************************************************************************************/
/*This work was partly supported by  the European Unions 7th Framework Programme for      */
/* research, technological development and demonstration under grant agreement             */
/*no. 602552, IDEAL'                                                                       */
/*******************************************************************************************/


/*******************************************************************************************/
/*  Design                                                                                 */
/*  It is assumed that two treatments are being compared (say, A & B) and that patients    */
/*  are randomised to treatments in pairs of periods referred to as cycles                 */
/*  In any cycle a given patient will receive A in one period and B in the other, the order*/
/*  being random.                                                                          */
/*******************************************************************************************/

/*******************************************************************************************/
/*  Assumptions. For the purpose of planning it is assumed that each of n patients will    */
/*  be treated in k cycles so that there are 2 x k x n treatment episodes of treatment     */
/*  altogether.                                                                            */
/* Two variance components are assumed relevant to determining efficiency of any designs   */
/* 1) The within patient variance                                                          */
/* 2) The variance of the random treatment-by-patient interaction                          */
/*******************************************************************************************/

/*******************************************************************************************/
/* Purpose of program                                                                      */
/* The program allows the user to investigate the value of n for given k that will produce */
/* the desired power for various possible analyses                                         */
/* The user sets the maximum number of cycles K_max and the minimum K_min & the            */
/*  sample size n will be calculated for every value from k_min to k_max                   */
/*  standard errors of naive and shrunk estimates will also be calculated                  */
/*  as will standard errors of ratios of weights for shrunk estimates                      */
/*  various plots will be produced                                                         */
/*  sample size n will be calculated for every value from k_min to k_max                   */
/*******************************************************************************************/



/*******************************************************************************************/
/*  Theory is covered in the following paper:                                              */
/* Sample size considerations for n-of-1 trials, by Stephen Senn                           */
/*******************************************************************************************/

/*******************************************************************************************/
/* Begin programming                                                                       */
/*******************************************************************************************/
/* Define data set to hold parameter values                                                */
/*******************************************************************************************/

data parameters; *data step to hold set parameters;
  label PSI='random effect SD'
		SIGMA='within cycle SD'
		EFFECT='clinically relevant difference'
		ALPHA= 'type I error rate'
		BETA ='type II error rate'
        K_max='maximum number of cycles'
		K_min='minimum number of cycles'
		C= 'control parameter'
		SIDES='sides for test'
        MinN='Minimum number of patients for SE of ratio of weights'
        MaxN='Maximum number of patents for SE of ratio of weights'
        K2='number of cycles for ratio of weights calculations'; 
/*******************************************************************************************/
/* Set control parameter C                                                                 */
/* C=0 if SIGMA = standard deviation of differences between repeat measures                */
/* C=1 if SIGMA = standard deviation of within-patient within cycle errors                 */
/*******************************************************************************************/

   C=1;

/*******************************************************************************************/
/* set parameter values                                                                    */
/*******************************************************************************************/


  EFFECT=1; *clinically relevant difference;
  SIGMA=2;  *Meaning depends on value of C. See above;
  PSI=1; *SD of random treatment-by-patient interaction;
  ALPHA=0.05; *type I error rate;
  BETA=0.20; *type II erorr rate;
  K_max=15; *maximum number of cycles;
  K_min=2; *minimum number of cycles must be at least 2;
  SIDES=2; *number of sides for test, 1 = one-sided, 2= two-sided;
  MinN=20; *Minimum number of patients for SE of ratio of weights;
  MaxN=100; *Maximum number of patients for SE of ratio of weights;
  K2=3;*number of cycles for ratio of weights calculations;
  
  
/*******************************************************************************************/
/*derived parameters                                                                       */
/*******************************************************************************************/

   SIGMA2=SIGMA*(sqrt(2))**C; *will multiply sigma by root(2) if C=1 else leave it unchanged;
   Z1=probit(1-ALPHA/SIDES); *z value for Type I error;
   Z2=probit(1-BETA);*Z value for type 2 error;    
   output;
   
run;

/*******************************************************************************************/
/* Begin calculation                                                                       */
/*******************************************************************************************/


/*******************************************************************************************/
/*Fixed effects approach                                                                   */
/*******************************************************************************************/

data fixed; *data step for fixed effects calculations;
 set parameters; *include values from data step parameters;
 label  DF='degrees of freedom'
        NF='number of patients'
        T1='critical value'
        SIGMA_fixed='SD of patient contrast'
        K= 'number of cycles'
        POWERF='Power'
        NKF='total number of cycles';
 do I=K_min to K_max; *loop for number of cycles;
    K=I;*current value for number of cycles;
    SIGMA_fixed=SQRT(SIGMA2**2/K); *SD for one patient";
    NF=((Z1+Z2)*(SIGMA_fixed/EFFECT))**2; *approximate sample size;
    NF=2*(int(NF/2-1)); *round down to integer value;
    DF=NF*(K-1); *degrees of freedom. Note that these depend on cycles also;
    T1=tinv(1-ALPHA/SIDES,DF); *critical quantile of the t-distribution;
    NONCENT=EFFECT/(SIGMA_fixed/sqrt(NF)); *non-centrality parameter;
    POWERF=100*(1-probt(T1,DF,NONCENT));*fixed effect approach power using non-central t;
    NKF=NF*K;*calculate total number of cycles;
    do while(POWERF<(100*(1-beta))); *increase sample size until target POWERF is reached;
       NF=NF+1; *increase sample size by 1;
       DF=NF*(K-1); *degrees of freedom. Note that these depend on cycles also;
       T1=tinv(1-ALPHA/SIDES,DF);*critical quantile of the t-distribution;
       NONCENT=EFFECT/(SIGMA_fixed/sqrt(NF)); *calculate non-centrality parameter;
       POWERF=100*(1-probt(T1,DF,NONCENT));*fixed effect approach;	   
       NKF=NF*K;*calculate total number of cycles;
    end;
    output;
  end;
run;
/*******************************************************************************************/
/*Random effects approach                                                                  */
/*******************************************************************************************/

data random;*data step for random effects calculations;
set parameters; *include values from data step parameters;
label DF='degrees of freedom'
      NR='number of patients'
      T1='critical value'
      SIGMA_random='SD of patient contrast'
      K= 'number of cycles'
      POWERR='power'
	  NKR='total number of cycles';

do I=K_min to K_max;*loop for number of cycles;
   K=I;*current value for number of cycles;
   SIGMA_random=SQRT(PSI**2+SIGMA2**2/K); *SD for one patient";
   NR=((Z1+Z2)*(SIGMA_random/EFFECT))**2; *approximate sample size;
   NR=2*(int(NR/2+1)); *round up to integer value;
   DF=NR-1; *degrees of freedom;
   T1=tinv(1-ALPHA/SIDES,DF);*critical quantile of the t-distribution;
   NONCENT=EFFECT/(SIGMA_random/sqrt(NR));*non-centrality parameter;
   POWERR=100*(1-probt(T1,DF,NONCENT));*random effect approach;
   NKR=NR*K;*calculate total number of cycles;
   do while(POWERR<(100*(1-beta)));*increase sample size until target POWERR is reached;
      NR=NR+1; *increase sample size by 1;
      DF=NR-1;*degrees of freedom;
      T1=tinv(1-ALPHA/SIDES,DF);*critical quantile of the t-distribution;
      NONCENT=EFFECT/(SIGMA_random/sqrt(NR)); *calculate non-centrality parameter;
      POWERR=100*(1-probt(T1,DF,NONCENT));*random effect approach;
      NKR=NR*K;*calculate total number of cycles;
   end;
   output;   
 end;
run;

/*******************************************************************************************/
/*Number of cycles for a future patient                                                    */
/* NB it is assumed that the nuisance parameter values have been estimated                 */
/* with high precision                                                                     */
/*******************************************************************************************/

data future;*datastep for future patient;
set parameters; *include values from data step parameters;
label SIGMA_future='SE for shrunk estimate'
      NAIVE='SE for naive estimate';
do I=K_min to K_max;
   K=I;
   NAIVE=SIGMA2/SQRT(K);
   SIGMA_future=SQRT((PSI**2)*(SIGMA2**2)/(K*(PSI**2)+SIGMA2**2)); *SD for one patient";
   NONCENT=EFFECT/SIGMA_future;*non-centrality for new patient;
   Z=Z1-NONCENT;
   POWER=100*(1-probnorm(Z));
   REFERENCE=PSI;
   output;   
  end;
run;

/*******************************************************************************************/
/*  Merged data set for fixed and random effects analysis                                  */
/*******************************************************************************************/
data both;
 label N='number of patients'
       NK='total number of cycles';
 merge fixed random;*datasteps for fixed and random effects;
 by K;
 keep K DF NR NF POWERF POWERR POWER N NK type NKF NKR;
 type ='fixed';
 POWER=powerF;
 N=NF;
 NK=NKF;
 output;
 type ='random';
 POWER=powerR;
 N=NR;
 NK=NKR;
 output;
run;
Proc print data=both;
run;
*Data set to perform calculations for SE of ratio of weights;

data ratio;*data step for ratio of weights calculations;
set parameters; *include values from data step parameters;
label SIGMA_random='SD of patient contrast'
      SE= 'standard error of ratio of weights'
	  VRat = 'variance of ratio of weights'
      RATIO='ratio of weights'
	  NP='number of patients';
	  Ratio=(Psi**2)*K2/(2*(Sigma**2));


do I=MinN to MaxN;*loop for number of patients;
   NP=I;*current value for number of patients;
   term1=(((K2*(PSI**2)/(2*(SIGMA**2)))+1)**2);
   term2=(2*(NP*(K2-1))**2)*(NP*K2-3);
   term3=((NP-1)*((NP*(K2-1)-2)**2)*(NP*(K2-1)-4));
   VRat=term1*term2/term3;
   SE=VRat**0.5;
   output;   
 end;
run;
/*******************************************************************************************/
/*  print results of calculations                                                          */
/*******************************************************************************************/

TITLE 'Sample size for a set of n of 1 trials run in K cycles';
TITLE2 'Parameter settings';
FOOTNOTE 'If control=1 within SD is for within-cycle errors. If control=0 within SD is for within-cycle differences';
/*******************************************************************************************/
/*  print parameter setings                                                                */
/*******************************************************************************************/

proc print label data=parameters;
  var ALPHA BETA SIDES EFFECT PSI SIGMA K_max C;
run;

/*******************************************************************************************/
/*  print fixed effects results                                                            */
/*******************************************************************************************/

TITLE2 'Power calculation: fixed effects approach';
FOOTNOTE '';
proc print label data=fixed;
  var SIGMA_fixed NF K DF T1 NKF POWERF NONCENT;
run;
/*******************************************************************************************/
/*  print random effects results                                                           */
/*******************************************************************************************/

TITLE2 'Power calculation: random effects approach';
FOOTNOTE '';
proc print label data=random;
  var SIGMA_random NR K DF T1 NKR POWERR NONCENT;
run;
/*******************************************************************************************/
/*  print calculation for future patient                                                   */
/*******************************************************************************************/
 TITLE2 'Naive and Shrunk standard errors & Power calculation: future patient';
 FOOTNOTE '';
 proc print label data=future;
     var K naive SIGMA_future NONCENT POWER;
 run;
/*******************************************************************************************/
/*  print calculation for ratio of weights                                                 */
/*******************************************************************************************/
 TITLE2 'Standard errors for ratios of weights from shrunk estimates';
 FOOTNOTE '';
 proc print label data=ratio;
     var K2 NP SE Ratio;
 run;


/*******************************************************************************************/
/*  plot results of calculations                                                           */
/*******************************************************************************************/
*set graph axes;
 *use proc means to find maximum values of axes;
 proc means max noprint data=both;
   var k NR NKR;
   output out=axis_1 max=k_max n_max nk_max;
 run;
proc means max noprint data=future;
   var naive;
   output out=axis_2 max=naive_max;
 run;
proc means min max noprint data=ratio;
   var NP SE;
   output out=axis_3 max=NP_max SE_max min=NP_min SE_min;
 run;

*deafine macros variables to set axes;
data axis;
   merge axis_1 axis_2 axis_3;   
   Ymax1=1.1*n_max;
   Ymax2=1.1*nk_max;
   Ymax3=1.1*naive_max;
   ymax4=1.1*SE_max;
   Xmax1=k_max+1;
   call symput('YU1',Ymax1);*macro variable for Y axis for fixed and random effects;
   call symput('YU2',Ymax2);*macro variable for Y axis for plots for fixed and random effects;
   call symput('YU3',Ymax3);*macro variable for Y axis for plots for SEs for future patient;
   call symput('YU4',Ymax4);*macro variable for Y axis for plots for SEs for weight ratios;
   call symput('XU',Xmax1);*macro variable for X axis for most plots;
   call symput('XU2',NP_max);*macro variable for X axis for plots for SEs for ratios of weights;
   call symput('XL2',NP_min);*macro variable for X axis for plots for SEs for ratios of weights;
   output;
 run;  
 
axis1 minor=none order=(0 to &XU by 1); *define X axis;
axis2 minor=(number=10) order= (0 to &YU1 by 10); *define Y axis 1;
axis3 order= (0 to &YU2 by 10); *define Y axis 2;
axis4 minor=(number=10) order= (0 to &YU3) label=(a=90 'Standard Error'); *define Y axis 3;
axis5 minor=(number=10)order=(&XL2 to &XU2 by 10) label=('Number of patients');*define X axis 2;
axis6 minor=(number=10) order= (0 to &YU4 by 0.05)  label=(a=90 'Standard Error');*define Y axis 4;

/*******************************************************************************************/
/*  fixed effects plot                                                                     */
/*******************************************************************************************/

TITLE2 'Fixed effects analysis';
proc gplot data=fixed;   
  symbol1 v = dot c=blue i=join;
  plot NF*K / vaxis=axis2 haxis=axis1;
run;

/*******************************************************************************************/
/*  random effects plot                                                                    */
/*******************************************************************************************/
TITLE2 'Random effects analysis';
proc gplot data=random;
  symbol1 v = diamondfilled c=red i=join;
  plot NR*K / vaxis=axis2 haxis=axis1;
run;
TITLE2 'Fixed and random effects analysis';
/*******************************************************************************************/
/*  fixed and random effects plot                                                          */
/*******************************************************************************************/

proc gplot data=both;
  symbol1 v = dot c=blue i=join;
  symbol2 v = diamondfilled c=red i=join;
  plot N*K=type / vaxis=axis2 haxis=axis1;
run;
/*******************************************************************************************/
/*  Total numbers of cycles                                                         */
/*******************************************************************************************/
TITLE1 'Set of n of 1 trials';
TITLE2 'Total numbers of cycles';
proc gplot data=both;
  symbol1 v = dot c=blue i=join;
  symbol2 v = diamondfilled c=red i=join;
  plot NK*K=type / vaxis=axis3 haxis=axis1;
run;


/*******************************************************************************************/
/*  Naive and shrunk standard errors                                                       */
/*******************************************************************************************/
TITLE1 'Set of n of 1 trials';
TITLE2 'Naive and shrunk standard errors';
proc gplot data=future;   
  symbol1 v = dot c=blue i=join;
  symbol2 v= dot c=red i=join;
  symbol3 c=none c=black i=join line=2;
  plot SIGMA_future*K=2 NAIVE*k=1 REFERENCE*k=3/ vaxis=axis4 haxis=axis1 overlay;
run;

/*******************************************************************************************/
/*  standard errors or ratios of weights                                                   */
/*******************************************************************************************/
TITLE1 'Set of n of 1 trials';
TITLE2 'standard errors of ratios of weights';
proc gplot data=ratio;   
  symbol2 v= dot c=red i=join;
  symbol3 c=none c=black i=join line=2;
  plot SE*NP=2 Ratio*NP=3/ vaxis=axis6 haxis=axis5 overlay;
run;

